"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useBalances = void 0;
const contexts_1 = require("../contexts");
const objects_1 = require("../interfaces/objects");
const react_1 = require("react");
const react_alert_1 = require("react-alert");
const constants_1 = require("../constants");
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHING"] = "FETCHING";
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["ERROR"] = "ERROR";
    EActionTypes["TOKEN_SELECTED"] = "TOKEN_SELECTED";
    EActionTypes["STATE_CHANNEL_CHANGED"] = "STATE_CHANNEL_CHANGED";
})(EActionTypes || (EActionTypes = {}));
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
function useBalances() {
    const { coreProxy, UIEvents } = contexts_1.useStoreContext();
    const { setLoading } = contexts_1.useLayoutContext();
    const alert = react_alert_1.useAlert();
    const initialState = {
        error: false,
        loading: true,
        balances: [],
        balancesByChannelAddress: [],
        balancesByChannelAddresses: new Map(),
        channelTokenSelectorOptions: [],
        preferredPaymentToken: undefined,
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHING:
                return {
                    ...state,
                    error: false,
                    loading: true,
                };
            case EActionTypes.FETCHED:
                return {
                    ...state,
                    error: false,
                    loading: false,
                    balances: prepareBalances(action.payload),
                    balancesByChannelAddresses: prepareBalancesByChannelAddresses(action.payload),
                    channelTokenSelectorOptions: prepareChannelTokenSelectorOptions(action.payload),
                };
            case EActionTypes.STATE_CHANNEL_CHANGED:
                return {
                    ...state,
                    error: false,
                    loading: false,
                    balancesByChannelAddress: prepareBalancesByChannelAddress(action.payload),
                };
            case EActionTypes.TOKEN_SELECTED:
                return {
                    ...state,
                    error: false,
                    loading: false,
                    preferredPaymentToken: action.payload,
                };
            case EActionTypes.ERROR:
                return {
                    ...state,
                    error: true,
                    loading: false,
                };
            default:
                return state;
        }
    }, initialState);
    react_1.useEffect(() => {
        let cancelRequest = false;
        const fetchData = async () => {
            if (cancelRequest)
                return;
            dispatch({ type: EActionTypes.FETCHING });
            setLoading(true);
            coreProxy === null || coreProxy === void 0 ? void 0 : coreProxy.getBalances().map((balance) => {
                setLoading(false);
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: balance,
                });
            }).mapErr((error) => {
                setLoading(false);
                alert.error(error.message || "An error has happened while pulling balances");
                dispatch({ type: EActionTypes.ERROR, payload: error.message });
            });
        };
        fetchData();
        coreProxy === null || coreProxy === void 0 ? void 0 : coreProxy.onBalancesChanged.subscribe({
            next: (balance) => {
                if (cancelRequest)
                    return;
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: balance,
                });
            },
        });
        UIEvents.onSelectedStateChannelChanged.subscribe({
            next: (activeStateChannel) => {
                if (cancelRequest)
                    return;
                coreProxy === null || coreProxy === void 0 ? void 0 : coreProxy.getBalances().map((balance) => {
                    setLoading(false);
                    dispatch({
                        type: EActionTypes.FETCHED,
                        payload: balance,
                    });
                }).map(() => {
                    dispatch({
                        type: EActionTypes.STATE_CHANNEL_CHANGED,
                        payload: activeStateChannel,
                    });
                });
            },
        });
        return function cleanup() {
            cancelRequest = true;
        };
    }, []);
    function prepareBalances(balance) {
        return balance.assets.reduce((acc, assetBalance) => {
            acc.push(assetBalance);
            return acc;
        }, []);
    }
    function prepareBalancesByChannelAddresses(balance) {
        return balance.assets.reduce((acc, assetBalance) => {
            acc.set(assetBalance.channelAddress, [
                ...(acc.get(assetBalance.channelAddress) || []),
                assetBalance,
            ]);
            return acc;
        }, new Map());
    }
    function prepareBalancesByChannelAddress(stateChannel) {
        return state.balances.filter((assetBalance) => assetBalance.channelAddress === stateChannel.channelAddress);
    }
    function prepareChannelTokenSelectorOptions(balance) {
        return balance.assets.reduce((acc, assetBalance) => {
            const tokenName = assetBalance.assetAddress === constants_1.ETHER_HEX_ADDRESS
                ? "ETH"
                : "HyperToken";
            acc.push(new objects_1.PaymentTokenOptionViewModel(tokenName, assetBalance.assetAddress));
            return acc;
        }, []);
    }
    return {
        ...state,
    };
}
exports.useBalances = useBalances;
//# sourceMappingURL=useBalances.js.map